#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>

#include "struct.h"
#include "init.h"
#include "daemon.h"
#include "client.h"

#ifdef DOJDEBUG
#include "debug.h"
void dojsig(int s)
{
  PRINTF("pid=%i ", getpid());
  switch(s)
    {
    case SIGHUP : PRINTF("Hangup (POSIX).\n"); break;
    case SIGINT : PRINTF("Interrupt (ANSI).\n"); break;
    case SIGQUIT : PRINTF("Quit (POSIX).\n"); break;
    case SIGILL : PRINTF("Illegal instruction (ANSI).\n"); break;
    case SIGTRAP : PRINTF("Trace trap (POSIX).\n"); break;
    case SIGABRT : PRINTF("Abort (ANSI).\n"); break;
    case SIGBUS : PRINTF("BUS error (4.2 BSD).\n"); break;
    case SIGFPE : PRINTF("Floating-point exception (ANSI).\n"); break;
    case SIGKILL : PRINTF("Kill, unblockable (POSIX).\n"); break;
    case SIGUSR1 : PRINTF("User-defined signal 1 (POSIX).\n"); break;
    case SIGSEGV : PRINTF("Segmentation violation (ANSI).\n"); break;
    case SIGUSR2 : PRINTF("User-defined signal 2 (POSIX).\n"); break;
    case SIGPIPE : PRINTF("Broken pipe (POSIX).\n"); break;
    case SIGALRM : PRINTF("Alarm clock (POSIX).\n"); break;
    case SIGTERM : PRINTF("Termination (ANSI).\n"); break;
    case SIGSTKFLT : PRINTF("Stack fault.\n"); break;
    case SIGCHLD : PRINTF("Child status has changed (POSIX).\n"); break;
    case SIGCONT : PRINTF("Continue (POSIX).\n"); break;
    case SIGSTOP : PRINTF("Stop, unblockable (POSIX).\n"); break;
    case SIGTSTP : PRINTF("Keyboard stop (POSIX).\n"); break;
    case SIGTTIN : PRINTF("Background read from tty (POSIX).\n"); break;
    case SIGTTOU : PRINTF("Background write to tty (POSIX).\n"); break;
    case SIGURG : PRINTF("Urgent condition on socket (4.2 BSD).\n"); break;
    case SIGXCPU : PRINTF("CPU limit exceeded (4.2 BSD).\n"); break;
    case SIGXFSZ : PRINTF("File size limit exceeded (4.2 BSD).\n"); break;
    case SIGVTALRM : PRINTF("Virtual alarm clock (4.2 BSD).\n"); break;
    case SIGPROF : PRINTF("Profiling alarm clock (4.2 BSD).\n"); break;
    case SIGWINCH : PRINTF("Window size change (4.3 BSD, Sun).\n"); break;
    case SIGIO : PRINTF("I/O now possible (4.2 BSD).\n"); break;
    case SIGPWR : PRINTF("Power failure restart (System V).\n"); break;
    case SIGSYS : PRINTF("Bad system call.\n"); break;
    default: PRINTF("unknown signal# %i\n", s); break;
    }
  exit(1);
}
#endif

int main (int argc, const char ** argv) {
  sladd_config_t * config;

#ifdef DOJDEBUG
#if 0
  int i;
  for(i=0; i!=_NSIG; ++i)
    signal(i, dojsig);
#endif
  openlog("sladd", LOG_PID, LOG_LOCAL0);
#endif

  if (geteuid() == 0 && getuid() != 0) {
    if(setuid(0) < 0)
      perror("could not become root. Perhaps suid bit was lost?");
    if(setgid(0) < 0)
      perror("could not set group to root. Perhaps suid bit was lost?");
  }

  config = init (argc, argv);
  if (!config)
    {
      fprintf(stderr, "could not init config\n");
      return 1;
    }

  if (config->command == 'd')
    return start_daemon (config);

  return run_client (config);
}
