#ifndef __struct_h_included__
#define __struct_h_included__

#include <sys/types.h>

/*
 * Return messages for "run" command
 */
#define MSG_OK        "200 queued"
#define MSG_SHUTDOWN  "200 terminating\n"
#define MSG_CANTQUEUE "201 can't queue\n"
#define MSG_NOCOMMAND "401 unknown command\n"
#define MSG_ALREADY   "403 plugin already running\n"
#define MSG_NOPLUGIN  "404 plugin not found\n"
#define MSG_CANTRUN   "500 internal error"
#define MSG_MAXCONNS  "501 Connection limit reached\n"

#define RECVBUFSIZE 1024

/*
 * A plugin entry is the atom of a slad plugin. It contains the necessary
 * information to run a plugin.
 */

typedef struct slad_plugin_entry {
  char * name;                          /* name of the entry */
  char * desc;                          /* description of the entry */
  int is_default;                       /* this entry is selected by default */
  char * binary;                        /* location of the binay,
                                           relative to the plugin's path */
  char * cmdline;                       /* parameters to pass to the binary */
  int max_runtime;                      /* maximum time in seconds the process
					   may run */
  int max_loglines;                     /* maximum number of lines to log */
  char * runasuser;                     /* the user and group to change to */
  char * runasgroup;                    /* before starting the command */
} slad_plugin_entry_t;

/*
 * A plugin sets contains a list of entries that can be run in parallel.
 * These can be selected or deselected by the client.
 */

typedef struct slad_plugin_set {
  char * name;                          /* name of the set */
  char * desc;                          /* description of the set */
  int is_default;                       /* this set is the default */
  struct slad_plugin_entry ** entries;  /* NULL -terminated list of entries */
} slad_plugin_set_t;

/*
 * A plugin contains a list of sets. Sets are mutually exclusive and can
 * optionally contain a single default set.
 */

typedef struct slad_plugin {
  char * name;                          /* name of the plugin */
  char * desc;                          /* plugin description */
  char * path;                          /* location of plugin files,
					   relative to slad directory */
  struct slad_plugin_set ** sets;       /* NULL-terminated list of sets */
} slad_plugin_t;

/*
 * Runtime configuration for the sladd.
 */

typedef struct sladd_config {
  char * pluginpath;                    /* path to the plugins */
  int listenport;                       /* TCP port to listen when running as
					   daemon */
  char command;                         /* command from cmd line */
  char * arg;                           /* cmd line argument */
  slad_plugin_t ** plugins;             /* list of plugins */
} sladd_config_t;

/*
 * Information about a job
 */

typedef struct sladd_job {
  pid_t pid;                            /* Pid the job is running at */
  int running;                          /* 0=completed, 1=forked, 2=exec'ed */
  int socket;                           /* The socket for reading data */
  char * pluginid;                      /* The plugin this belongs to */
  char * output;                        /* The output the program had */
  int outputlen;                        /* Length of output */
  struct sladd_job * next;              /* For linked list usage */
} sladd_job_t;

/*
 * A structure for the information related to command connections
 */

typedef struct receive_buffer {
  int socket;                           /* The socket accept() gave */
  char recvbuf[RECVBUFSIZE];            /* A receive buffer in case of fragments */
  int recvlen;                          /* The number of characters buffered */
  time_t last_event;                    /* time() of the last event on the socket */
} receive_buffer_t;

#endif
