#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "debug.h"
#include "struct.h"

void reuse (int socket) {
  int on = 1;
  setsockopt (socket, SOL_SOCKET, SO_REUSEADDR, (void *) &on, sizeof (on));
}

int enqueue (void *** array, void * data) {
  int elements = 0;
  void ** tmp;

  if (*array)
    for (elements = 0; array[0][elements]; elements++);
  tmp = realloc (*array, (elements+2) * sizeof (void *));
  if (tmp) {
    *array = tmp;
    (*array)[elements] = data;
    (*array)[elements+1] = NULL;
    return 0;
  } else
    return 1;
}

void findplugin (const char * command, sladd_config_t * config,
		 slad_plugin_t ** plugin, slad_plugin_set_t ** set,
		 slad_plugin_entry_t ** entry) {
  char * tmpstring, * pname, * sname, * ename = NULL;
  int p_num = 0, s_num = 0, e_num = 0;
  int len;

  *plugin = NULL;
  *set = NULL;
  *entry = NULL;

  if (!(config->plugins))
    {
      PRINTF("findplugin(): could not find any plugins\n");
      return;
    }

  pname = tmpstring = strdup (command);
  if (!pname)
    {
      PRINTF("findplugin(): could not strdup command\n");
      return;
    }

  len = strlen (pname);
  // strip white spaces at the beginning
  while (*pname && isspace(*pname)) {
    len--;
    pname++;
  }
  // strip white spaces at the end
  while (len && isspace (pname[len-1])) {
    len--;
    pname[len] = 0;
  }

  sname = strchr (pname, ':');
  if (sname) {
    *sname = 0;
    sname++;

    ename = strchr (sname, ':');
    if (ename) {
      *ename = 0;
      ename++;
    }
  }

  for (p_num = 0; (*plugin = config->plugins[p_num]); p_num++)
    if (!strcasecmp ((*plugin)->name, pname)) {
      if (!sname)
	return;
      for (s_num = 0; (*set = (*plugin)->sets[s_num]); s_num++)
	if (!strcasecmp ((*set)->name, sname)) {
	  if (!ename) return;
	  for (e_num = 0; (*entry = (*set)->entries[e_num]); e_num++)
	    if (!strcasecmp ((*entry)->name, ename)) {
	      free (tmpstring);
	      return;
	    }
	}
    }

  free (tmpstring);
}
